# -*- coding: utf-8 -*-

# Author: Alexandre Boucaud <aboucaud@lal.in2p3.fr>
# License: BSD 3 clause

from __future__ import division, print_function

import os
import time
import logging
import numpy as np
import keras.backend as K

from astroblending.data import read_train_data, read_test_data
from astroblending.detector import ObjectDetector

K.tf.set_random_seed(0)

bench_name = os.environ.get('ASTRO_BENCH_NAME')
output_path = os.environ.get('ASTRO_OUTPUT_PATH')
dataset_path = os.environ.get('ASTRO_DATASET_PATH')
num_gpus = int(os.environ.get('ASTRO_NUM_GPUS'))

logging.basicConfig(filename=f'{output_path}/{bench_name}.log', level=logging.INFO, format='%(asctime)s [ %(levelname)s ] : %(message)s')
logger = logging.getLogger(bench_name)

logger.info("Tensorflow version " + K.tf.__version__)

filename = os.path.splitext(os.path.basename(__file__))[0]

logger.info("Reading data...")

X_train, Y_train = read_train_data(dataset_path)
X_test, Y_test = read_test_data(dataset_path)

logger.info("Creating object detector...")
obj = ObjectDetector(epoch=50, num_gpus=num_gpus, model_check_point=True, filename=filename, plot_history=True,
                    output_path=output_path, bench_name=bench_name)

t0 = time.time()
logger.info("Training...")
obj.fit(X_train, Y_train)

t1 = time.time()
logger.info("Testing...")
score = obj.predict_score(X_test, Y_test)

t2 = time.time()

traintime = t1 - t0
testtime = t2 - t1

logger.info("")
logger.info("\tResults")
logger.info("\t=======")
logger.info(f"\ttrain_time: {traintime:.1f} sec - {traintime / 60:.1f} min")
logger.info(f"\ttest_time: {testtime:.1f} sec - {testtime / 60:.1f} min")
logger.info(f"\tscore: {score*100:.2f}/100")

# Results file
resfile = os.path.join(output_path, 'results.csv')
header = ','.join(["bench_name", "train_time", "test_time", "score"])
scoreline = f"{bench_name},{traintime:.2f},{testtime:.2f},{score}"

if not os.path.exists(resfile):
    with open(resfile, 'a') as f:
        print(header, file=f)

with open(resfile, 'a') as f:
    print(scoreline, file=f)
